@@ -1,63 +1,35 @@
-/****************************************************************************
-**
-** Copyright (C) 2017-2018 Red Hat, Inc
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+/*
+
+    SPDX-FileCopyrightText: 2017-2018 Red Hat Inc
+    Contact: https://www.qt.io/licensing/
+
+    This file is part of the plugins of the Qt Toolkit.
+
+    SPDX-License-Identifier: LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KFQF-Accepted-GPL OR LicenseRef-Qt-Commercial
+
+*/
 
 #include "qxdgdesktopportalfiledialog_p.h"
 
-#include <QtCore/qeventloop.h>
+#include <qeventloop.h>
 
-#include <QtDBus/QtDBus>
 #include <QDBusConnection>
 #include <QDBusMessage>
 #include <QDBusPendingCall>
 #include <QDBusPendingCallWatcher>
 #include <QDBusPendingReply>
+#include <QtDBus>
 
 #include <QFile>
 #include <QMetaType>
-#include <QMimeType>
 #include <QMimeDatabase>
+#include <QMimeType>
 #include <QRandomGenerator>
 #include <QWindow>
 
 QT_BEGIN_NAMESPACE
 
-QDBusArgument &operator <<(QDBusArgument &arg, const QXdgDesktopPortalFileDialog::FilterCondition &filterCondition)
+QDBusArgument &operator<<(QDBusArgument &arg, const QXdgDesktopPortalFileDialog::FilterCondition &filterCondition)
 {
     arg.beginStructure();
     arg << filterCondition.type << filterCondition.pattern;
@@ -65,7 +37,7 @@ QDBusArgument &operator <<(QDBusArgument &arg, const QXdgDesktopPortalFileDialog
     return arg;
 }
 
-const QDBusArgument &operator >>(const QDBusArgument &arg, QXdgDesktopPortalFileDialog::FilterCondition &filterCondition)
+const QDBusArgument &operator>>(const QDBusArgument &arg, QXdgDesktopPortalFileDialog::FilterCondition &filterCondition)
 {
     uint type;
     QString filterPattern;
@@ -78,7 +50,7 @@ const QDBusArgument &operator >>(const QDBusArgument &arg, QXdgDesktopPortalFile
     return arg;
 }
 
-QDBusArgument &operator <<(QDBusArgument &arg, const QXdgDesktopPortalFileDialog::Filter filter)
+QDBusArgument &operator<<(QDBusArgument &arg, const QXdgDesktopPortalFileDialog::Filter filter)
 {
     arg.beginStructure();
     arg << filter.name << filter.filterConditions;
@@ -86,7 +58,7 @@ QDBusArgument &operator <<(QDBusArgument &arg, const QXdgDesktopPortalFileDialog
     return arg;
 }
 
-const QDBusArgument &operator >>(const QDBusArgument &arg, QXdgDesktopPortalFileDialog::Filter &filter)
+const QDBusArgument &operator>>(const QDBusArgument &arg, QXdgDesktopPortalFileDialog::Filter &filter)
 {
     QString name;
     QXdgDesktopPortalFileDialog::FilterConditionList filterConditions;
@@ -104,7 +76,8 @@ class QXdgDesktopPortalFileDialogPrivate
 public:
     QXdgDesktopPortalFileDialogPrivate(QPlatformFileDialogHelper *nativeFileDialog)
         : nativeFileDialog(nativeFileDialog)
-    { }
+    {
+    }
 
     WId winId = 0;
     bool modal = false;
@@ -112,11 +85,11 @@ public:
     bool selectDirectory = false;
     bool saveFile = false;
     QString acceptLabel;
-    QString directory;
+    QUrl directory;
     QString title;
     QStringList nameFilters;
     QStringList mimeTypesFilters;
-    QStringList selectedFiles;
+    QList<QUrl> selectedFiles;
     QPlatformFileDialogHelper *nativeFileDialog = nullptr;
 };
 
@@ -176,7 +149,7 @@ void QXdgDesktopPortalFileDialog::openPortal()
                                                           QLatin1String("/org/freedesktop/portal/desktop"),
                                                           QLatin1String("org.freedesktop.portal.FileChooser"),
                                                           d->saveFile ? QLatin1String("SaveFile") : QLatin1String("OpenFile"));
-    QString parentWindowId = QLatin1String("x11:") + QString::number(d->winId);
+    QString parentWindowId = QLatin1String("x11:") + QString::number(d->winId, 16);
 
     QVariantMap options;
     if (!d->acceptLabel.isEmpty())
@@ -188,10 +161,10 @@ void QXdgDesktopPortalFileDialog::openPortal()
 
     if (d->saveFile) {
         if (!d->directory.isEmpty())
-            options.insert(QLatin1String("current_folder"), QFile::encodeName(d->directory).append('\0'));
+            options.insert(QLatin1String("current_folder"), QFile::encodeName(d->directory.toLocalFile()).append('\0'));
 
         if (!d->selectedFiles.isEmpty())
-            options.insert(QLatin1String("current_file"), QFile::encodeName(d->selectedFiles.first()).append('\0'));
+            options.insert(QLatin1String("current_file"), QFile::encodeName(d->selectedFiles.first().toLocalFile()).append('\0'));
     }
 
     // Insert filters
@@ -262,7 +235,7 @@ void QXdgDesktopPortalFileDialog::openPortal()
 
     QDBusPendingCall pendingCall = QDBusConnection::sessionBus().asyncCall(message);
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(pendingCall);
-    connect(watcher, &QDBusPendingCallWatcher::finished, this, [this] (QDBusPendingCallWatcher *watcher) {
+    connect(watcher, &QDBusPendingCallWatcher::finished, this, [this](QDBusPendingCallWatcher *watcher) {
         QDBusPendingReply<QDBusObjectPath> reply = *watcher;
         if (reply.isError()) {
             Q_EMIT reject();
@@ -272,7 +245,7 @@ void QXdgDesktopPortalFileDialog::openPortal()
                                                   QLatin1String("org.freedesktop.portal.Request"),
                                                   QLatin1String("Response"),
                                                   this,
-                                                  SLOT(gotResponse(uint,QVariantMap)));
+                                                  SLOT(gotResponse(uint, QVariantMap)));
         }
     });
 }
@@ -291,7 +264,7 @@ void QXdgDesktopPortalFileDialog::setDirectory(const QUrl &directory)
         d->nativeFileDialog->setDirectory(directory);
     }
 
-    d->directory = directory.path();
+    d->directory = directory;
 }
 
 QUrl QXdgDesktopPortalFileDialog::directory() const
@@ -301,7 +274,7 @@ QUrl QXdgDesktopPortalFileDialog::directory() const
     if (d->nativeFileDialog && (options()->fileMode() == QFileDialogOptions::Directory || options()->fileMode() == QFileDialogOptions::DirectoryOnly))
         return d->nativeFileDialog->directory();
 
-    return QUrl(d->directory);
+    return d->directory;
 }
 
 void QXdgDesktopPortalFileDialog::selectFile(const QUrl &filename)
@@ -313,7 +286,7 @@ void QXdgDesktopPortalFileDialog::selectFile(const QUrl &filename)
         d->nativeFileDialog->selectFile(filename);
     }
 
-    d->selectedFiles << filename.path();
+    d->selectedFiles << filename;
 }
 
 QList<QUrl> QXdgDesktopPortalFileDialog::selectedFiles() const
@@ -323,11 +296,7 @@ QList<QUrl> QXdgDesktopPortalFileDialog::selectedFiles() const
     if (d->nativeFileDialog && (options()->fileMode() == QFileDialogOptions::Directory || options()->fileMode() == QFileDialogOptions::DirectoryOnly))
         return d->nativeFileDialog->selectedFiles();
 
-    QList<QUrl> files;
-    for (const QString &file : d->selectedFiles) {
-        files << QUrl(file);
-    }
-    return files;
+    return d->selectedFiles;
 }
 
 void QXdgDesktopPortalFileDialog::setFilter()
@@ -367,8 +336,8 @@ void QXdgDesktopPortalFileDialog::exec()
 
     // HACK we have to avoid returning until we emit that the dialog was accepted or rejected
     QEventLoop loop;
-    loop.connect(this, SIGNAL(accept()), SLOT(quit()));
-    loop.connect(this, SIGNAL(reject()), SLOT(quit()));
+    QObject::connect(this, &QXdgDesktopPortalFileDialog::accept, &loop, &QEventLoop::quit);
+    QObject::connect(this, &QXdgDesktopPortalFileDialog::reject, &loop, &QEventLoop::quit);
     loop.exec();
 }
 
@@ -402,9 +371,15 @@ void QXdgDesktopPortalFileDialog::gotResponse(uint response, const QVariantMap &
     Q_D(QXdgDesktopPortalFileDialog);
 
     if (!response) {
-        if (results.contains(QLatin1String("uris")))
-            d->selectedFiles = results.value(QLatin1String("uris")).toStringList();
-
+        if (results.contains(QLatin1String("uris"))) {
+            const QStringList uris = results.value(QLatin1String("uris")).toStringList();
+            d->selectedFiles.clear();
+            d->selectedFiles.reserve(uris.size());
+            for (const QString &uri : uris) {
+                // uris are expected to have proper "file:" scheme set
+                d->selectedFiles.append(QUrl(uri));
+            }
+        }
         Q_EMIT accept();
     } else {
         Q_EMIT reject();
